
/**************************************************************************
 *
 *  $Id: mbg_serclock_cfg.h 1.4 2019/04/05 07:29:33Z martin.burnicki REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Definitions and prototypes for serial clock evaluation configuration.
 *
 * -----------------------------------------------------------------------
 *  $Log: mbg_serclock_cfg.h $
 *  Revision 1.4  2019/04/05 07:29:33Z  martin.burnicki
 *  Added some comments.
 *  Removed some obsolete code.
 *  Revision 1.3  2018/11/29 14:31:47  martin
 *  New preprocessor symbol MBG_SUPP_LANCLOCK to
 *  conditionally include lanclock stuff.
 *  Revision 1.2  2018/08/08 09:44:01Z  martin
 *  Renamed MBG_SERCLOCK_CFG::ref_offs_min to ref_offs_mins.
 *  Added MBG_LANCLOCK_CFG structure for XPORT devices.
 *  Revision 1.1  2011/09/23 13:56:32  martin
 *  Initial revision.
 *
 **************************************************************************/

#ifndef _MBG_SERCLOCK_CFG_H
#define _MBG_SERCLOCK_CFG_H

/* Other headers to be included */

#include <mbg_tgt.h>
#include <gpsdefs.h>
#include <pcpsdefs.h>
#include <pcpsutil.h>
#include <mbgtime.h>
#include <mbgpccyc.h>

#if !defined( MBG_SUPP_LANCLOCK )
  #define MBG_SUPP_LANCLOCK  0
#endif

#if MBG_SUPP_LANCLOCK
  #include <secudefs.h>
#endif

#if defined( MBG_TGT_WIN32 )
  #include <rs232.h>
  #include <mbgdevio.h>
#endif //MBG_TGT_WIN32

#include <use_pack.h>


#ifdef _MBG_SERCLOCK_CFG
 #define _ext
 #define _DO_INIT
#else
 #define _ext extern
#endif


/* Start of header body */

#if defined( _USE_PACK )
  #pragma pack( 1 )      // set byte alignment
  #define _USING_BYTE_ALIGNMENT
#endif


#ifdef __cplusplus
extern "C" {
#endif


#if !defined( _RS232_H )

// TODO: use a more appropriate type

typedef struct
{
  char ComName[20];   //##+++ modified, 7 in rs232.h under Windows
  char Framing[4];
  long Baudrate;
} COMSTRUCT;

#endif


typedef struct
{
  COMSTRUCT port;
  uint32_t telegram_type;
  uint32_t ref_type;
  uint32_t flags;
  MBG_REF_OFFS ref_offs_mins;

} MBG_SERCLOCK_CFG;


/**
 * @brief Bit masks used with MBG_SERCLOCK_CFG::flags
 */
enum
{
  /// This bit can be set if the applied time string
  /// always contains UTC time, even though this is not
  /// indicated by the statues character in the string.
  /// Can be configured in the monitor program.
  SERCLOCK_FORCE_UTC,

  /// If this bit is set then the serial port is not
  /// specifically configured to receive a time string,
  /// and characters are read from the receive buffer
  /// one after the other, which reduces the resulting
  /// accuracy.
  /// This option was introduced a long time ago
  /// (eventually under Windows NT) to work around reception
  /// problems in the default mode of operation.
  /// Must be set manually in the registry.
  SERCLOCK_READ_NORMAL,

  /// If the connected clock can sent the time string only
  /// once per minute then this flag can be set to avoid
  /// receive timeouts which would normally happen.
  /// Must be set manually in the registry.
  SERCLOCK_STRING_PER_MINUTE,

  N_SERCLOCK_FLAGS  //< The number of known bits.
};

#define MSK_SERCLOCK_FORCE_UTC         ( 1UL << SERCLOCK_FORCE_UTC )          //< see ::SERCLOCK_FORCE_UTC
#define MSK_SERCLOCK_READ_NORMAL       ( 1UL << SERCLOCK_READ_NORMAL )        //< see ::SERCLOCK_READ_NORMAL
#define MSK_SERCLOCK_STRING_PER_MINUTE ( 1UL << SERCLOCK_STRING_PER_MINUTE )  //< see ::SERCLOCK_STRING_PER_MINUTE



/* ----- function prototypes begin ----- */

/* This section was generated automatically */
/* by MAKEHDR, do not remove the comments. */

/* (No header definitions found) */

/* ----- function prototypes end ----- */

#ifdef __cplusplus
}
#endif


#if defined( _USING_BYTE_ALIGNMENT )
  #pragma pack()      // set default alignment
  #undef _USING_BYTE_ALIGNMENT
#endif

/* End of header body */

#undef _ext
#undef _DO_INIT

#endif  /* _MBG_SERCLOCK_CFG_H */
