
/**************************************************************************
 *
 *  $Id: mbgsvcio.h 1.18 2018/09/21 07:39:14Z martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Definitions and prototypes for mbgsvcio.c.
 *
 * -----------------------------------------------------------------------
 *  $Log: mbgsvcio.h $
 *  Revision 1.18  2018/09/21 07:39:14Z  martin
 *  New version code 308, compatibility version code still 200.
 *  New define USE_SERIAL_XHRT, still 0 by default.
 *  Updated function prototypes.
 *  Revision 1.17  2014/07/10 15:03:14Z  martin
 *  Moved some definitions here.
 *  Updated function prototypes.
 *  Revision 1.16  2009/08/14 09:28:13Z  daniel
 *  New version code 306, compatibility version code still 200.
 *  Revision 1.15  2009/06/09 08:57:47Z  daniel
 *  Rev No. 305
 *  Revision 1.14  2009/03/19 09:06:40Z  daniel
 *  New version code 304, compatibility version code still 200.
 *  Revision 1.13  2009/01/12 09:40:18Z  daniel
 *  New version code 303, compatibility version code still 200.
 *  Added comments in doxygen format.
 *  Revision 1.12  2008/01/17 10:14:41Z  daniel
 *  New version code 302, compatibility version code still 200.
 *  Revision 1.11  2007/10/16 10:16:27Z  daniel
 *  New version code 301, compatibility version code still 200.
 *  Revision 1.10  2007/09/24 15:28:17Z  martin
 *  New version code 300, compatibility version code still 200.
 *  Revision 1.9  2007/03/22 09:52:18Z  martin
 *  New version code 219, compatibility version code still 200.
 *  Removed obsolete headers.
 *  Revision 1.8  2006/08/09 13:38:02Z  martin
 *  New version code 218, compatibility version still 200.
 *  Revision 1.7  2006/06/08 12:23:54Z  martin
 *  New version code 217, compatibility version still 200.
 *  Revision 1.6  2006/05/02 12:52:17Z  martin
 *  New version code 216, compatibility version still 200.
 *  Revision 1.5  2006/01/11 12:04:32Z  martin
 *  New version code 215, compatibility version still 200.
 *  Revision 1.4  2005/12/15 09:16:38Z  martin
 *  New version code 214, compatibility version still 200.
 *  Revision 1.3  2005/07/20 07:38:39Z  martin
 *  New version code 213.
 *  Revision 1.2  2005/02/16 15:34:40Z  martin
 *  New version 2.12.
 *  Revision 1.1  2004/07/01 10:00:51Z  martin
 *
 **************************************************************************/

#ifndef _MBGSVCIO_H
#define _MBGSVCIO_H


/* Other headers to be included */

#include <mbg_tgt.h>
#include <words.h>
#include <pcpsdefs.h>



#ifdef _MBGSVCIO
  #define _ext
#else
  #define _ext extern
#endif


/* Start of header body */

#if defined( _USE_PACK )      // set byte alignment
  #pragma pack( 1 )
#endif

#define MBGSVCIO_VERSION         0x0308

#define MBGSVCIO_COMPAT_VERSION  0x0200


#define USE_SERIAL_XHRT    0  // not fully implemented/tested


#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief A union used to access a FILETIME as 64 bit number
 *
 * @note using this union may be tricky. Even though no problems
 * have been observed, yet, the MS docs clearly say there may be
 * alignment problems under 64 bit Windows.
 * If the FILETIME members are not 32 bit aligned then accessing
 * the dwl member will yield wrong values.
 */
typedef union
{
  FILETIME ft;   ///< FILETIME structure as provided by the Windows API
  uint64_t dwl;  ///< used to access as single 64 bit number

} FT_DWL;



/**
 * @brief A timestamp in Windows format, plus %UTC offset and clock status
 */
typedef struct
{
  FT_DWL ftdwl;               ///< time stamp, based on %UTC
  int32_t utc_offs;           ///< %UTC offset in [sec]
  PCPS_TIME_STATUS_X status;  ///< extended status, see ::PCPS_TIME_STATUS_X

} REF_TIME;



/**
 * @brief Error information retrieved from the time adjustment service
 */
typedef struct
{
  uint32_t code;   ///< error/status code, see ::ERR_INFO_CODES
  REF_TIME time;   ///< clock's time and status when ::code was updated

} ERR_INFO;


/**
 * @brief Codes used with ::ERR_INFO::code
 */
enum ERR_INFO_CODES
{
  MBG_SVC_NO_ERROR,              ///< No current error condition
  MBG_SVC_ERR_OPEN_CLOCK_DEV,    ///< Failed to open clock device
  MBG_SVC_ERR_OPEN_SERIAL_PORT,  ///< Failed to open serial port
  MBG_SVC_ERR_SAME_REF_TIME,     ///< Clock device always returns same time (hardware failure)
  MBG_SVC_ERR_REF_TIME_STEP,     ///< Time on the clock device has jumped, service stopped disciplining system time
  N_MBG_SVC_ERR_CODE             ///< the number of known codes 
};



/* ----- function prototypes begin ----- */

/* This section was generated automatically */
/* by MAKEHDR, do not remove the comments. */

 /**
 * @brief Get the version number of the precompiled DLL/shared object library.
 *
 * If this library is used as a DLL/shared object library then the version
 * number can be checked to see if the header files which are actually used
 * to build an application are compatible with the header files which have
 * been used to build the library, and thus the API function are called
 * in the correct way.
 *
 * @return The version number
 *
 * @see ::mbgsvcio_check_version
 * @see ::MBGSVCIO_VERSION defined in mbgsvcio.h.
 */
 _MBG_API_ATTR int _MBG_API mbgsvcio_get_version( void ) ;

 /**
 * @brief Check if the DLL/shared library is compatible with a given version.
 *
 * If this library is used as a DLL/shared object library then the version
 * number can be checked to see if the header files which are actually used
 * to build an application are compatible with the header files which have
 * been used to build the library, and thus the API functions are called
 * in the correct way.
 *
 * @param[in] header_version  Version number to be checked, should be ::MBGSVCIO_VERSION
 *                            from the mbgsvcio.h file version used to build the application
 *
 * @return ::MBG_SUCCESS if compatible, else ::MBG_ERR_LIB_NOT_COMPATIBLE
 *
 * @see ::mbgsvcio_get_version
 * @see ::MBGSVCIO_VERSION defined in mbgsvcio.h
 */
 _MBG_API_ATTR int _MBG_API mbgsvcio_check_version( int header_version ) ;

 /**
 * @brief Query the status of the Meinberg time adjustment service "mbgadjtm.exe"
 *
 * @return  1, if the service has the state "SERVICE_RUNNING", otherwise 0.
 */
 _MBG_API_ATTR int _MBG_API mbg_time_adjustment_active( void ) ;

 /**
 * @brief Read the ::ERROR_INFO structure from the Meinberg time adjustment service
 *
 * @param[out]  p  Pointer to an ::ERROR_INFO structure to be filled on success
 * 
 * @return ::MBG_SUCCESS on success, else one of the @ref MBG_ERROR_CODES
 */
 _MBG_API_ATTR int _MBG_API mbg_get_time_adjustment_err_info( ERR_INFO *p ) ;

 /**
 * @brief Check if the reference clock is accessible.
 *
 * @return  1 if the reference clock is accessible<br>
 *          0 if the reference time is invalid (all 0), or the device is inaccessible.<br>
 *          One of the @ref MBG_ERROR_CODES if the shared memory area is not accessible.
 */
 _MBG_API_ATTR int _MBG_API mbg_ref_time_accessible( void ) ;

 /**
 * @brief Read the current refclock's state from the time adjustment service.
 *
 * This function should be used preferably over ::mbg_get_ref_time_status,
 * which is deprecated since the return code from that function can't be
 * reliably classified as bit coded clock status code, or error number.
 *
 * @param[out]  p_status  Pointer to a ::PCPS_TIME_STATUS_X to be filled on success, or NULL.
 *
 * @return ::MBG_SUCCESS on success, else one of the @ref MBG_ERROR_CODES
 *
 * @see ::PCPS_TIME_STATUS_X
 */
 _MBG_API_ATTR int _MBG_API mbg_get_ref_time_status_ex( PCPS_TIME_STATUS_X *p_status ) ;

 /**
 * @brief Read the current refclock's state from the time adjustment service.
 *
 * @deprecated This function is deprecated since its return code can't
 * be reliably classified as bit coded clock status code, or error number.
 *
 * @return  A ::PCPS_TIME_STATUS_X status code, or -1 in case of an error.
 *
 * @see ::mbg_get_ref_time_status_ex
 * @see ::PCPS_TIME_STATUS_X
 */
 _MBG_API_ATTR int _DEPRECATED_BY( "mbg_get_ref_time_status_ex" ) _MBG_API mbg_get_ref_time_status( void ) ;


/* ----- function prototypes end ----- */

#ifdef __cplusplus
}
#endif

#if defined( _USE_PACK )   // set default alignment
  #pragma pack()
#endif

/* End of header body */

#undef _ext

#endif  // _MBGSVCIO_H

