
/**************************************************************************
 *
 *  $Id: mbgsvctl.h 1.30 2018/11/29 14:27:30Z martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Definitions and prototypes for servio.c
 *
 * -----------------------------------------------------------------------
 *  $Log: mbgsvctl.h $
 *  Revision 1.30  2018/11/29 14:27:30Z  martin
 *  Updated function prototypes.
 *  Revision 1.29  2018/09/21 11:12:49Z  martin
 *  New version code 308, compatibility version code still 214.
 *  Moved definitions of MBGADJTM_CFG, MBG_SERCLOCK_CFG
 *  and related stuff elsewhere.
 *  Updated function prototypes.
 *  Revision 1.28  2009/12/15 15:35:09Z  daniel
 *  New version code 307, compatibility version code still 214.
 *  Include mbgserio.h.
 *  Windows.h is now included elsewhere.
 *  Revision 1.27  2009/08/14 09:28:41  daniel
 *  New version code 306, compatibility version code still 214.
 *  Revision 1.26  2009/06/09 08:57:47Z  daniel
 *  Rev No. 305
 *  Revision 1.25  2009/03/19 09:07:26Z  daniel
 *  New version code 304, compatibility version code still 214.
 *  Revision 1.24  2009/01/28 10:38:15Z  daniel
 *  Added serial flag SERCLOCK_STRING_PER_MINUTE and
 *  bit mask MSK_SERCLOCK_STRING_PER_MINUTE.
 *  Revision 1.23  2009/01/12 10:14:23Z  daniel
 *  New version code 303, compatibility version code still 214.
 *  Changed UINT to uint32_t and LPTSTR to char*.
 *  Added MBGADJTM_CFG parameters allow_timestep_on_refclk_jump
 *  and allow_initial_timestep.
 *  Avoid including mbgdevio.h when MBG_OMIT_MBGDEVIO_H is defined.
 *  Updated function prototypes.
 *  Revision 1.22  2008/01/30 07:53:58Z  daniel
 *  New version code 302, compatibility version code still 214.
 *  Added new members to MBGADJTM_CFG structure to
 *  support service event handling.
 *  Revision 1.21  2007/10/16 10:09:51Z  daniel
 *  New file version 3,1,0,0, product version 3,1,0,0.
 *  New version code 301, compatibility version code still 214.
 *  Revision 1.20  2007/09/26 14:46:43Z  martin
 *  New version code 300, compatibility version code still 214.
 *  Identify selected device by hardware ID instead of device index.
 *  Removed obsolete references to ASCII log file.
 *  Updated function prototypes.
 *  Revision 1.19  2007/03/21 16:46:27Z  martin
 *  New version code 219, compatibility version code still 214.
 *  Changes due to renamed library symbols.
 *  Updated function prototypes.
 *  Revision 1.18  2006/10/31 16:24:09Z  martin
 *  Added a serial port flag which prevents the read thread  from
 *  using special timeout settings.
 *  Revision 1.17  2006/08/09 13:31:11Z  martin
 *  New version code 218, compatibility version code still 214.
 *  Revision 1.16  2006/06/08 10:36:10Z  martin
 *  New version code 217, compatibility version code still 214.
 *  Revision 1.15  2006/05/23 19:35:15Z  martin
 *  Updated function prototypes.
 *  Revision 1.14  2006/05/02 10:08:51Z  martin
 *  New version code 216, compatibility version code still 214.
 *  Revision 1.13  2006/01/11 11:45:05Z  martin
 *  New version code 215, compatibility version code still 214.
 *  Revision 1.12  2005/12/15 09:02:05Z  martin
 *  New version 214 and compatibility version 214.
 *  New service registry parameter "flags" which controls
 *  logging time adjustment details, and starting paused.
 *  Updated function prototypes.
 *  Revision 1.11.1.1  2005/11/28 17:08:34Z  martin
 *  Revision 1.11  2005/07/20 07:23:58Z  martin
 *  New version 213 and compatibility version 213.
 *  New service cfg parameters notification_limit and
 *  set_mm_timer.
 *  Updated function prototypes.
 *  Revision 1.10  2004/04/27 13:46:40Z  martin
 *  New member flags in MBG_SERCLOCK_CFG.
 *  Revision 1.9  2004/04/27 07:17:47Z  martin
 *  Changed type of MBG_SERCLOCK_CFG::ref_offs.
 *  Removed function prototypes that are used in exe-File only
 *  New definition MBGSVCIO_COMPAT_VERSION.
 *  Pack structures 1 byte aligned.
 *  Updated function protoypes.
 *  Revision 1.8  2003/12/22 15:55:57Z  martin
 *  New DLL interface version number.
 *  Updated function prototypes.
 *  Revision 1.7  2003/12/10 10:41:50Z  martin
 *  New DLL interface version number.
 *  Revision 1.6  2003/10/31 11:11:11Z  martin
 *  New DLL interface version number.
 *  New structures with config parameters.
 *  Updated function prototypes.
 *  Revision 1.5  2003/09/23 09:58:36Z  martin
 *  Updated function prototypes.
 *  Revision 1.4  2003/04/09 15:24:01Z  martin
 *  Module renamed from servio.h.
 *  Include standard names and reg paths.
 *  Updated function prototypes.
 *  Revision 1.3  2002/09/30 07:54:35Z  martin
 *  Updated function prototypes by MAKEHDR.
 *  Revision 1.2  2002/02/22 10:35:11Z  Daniel
 *  Revision 1.1  2002/02/18 10:52:18Z  Daniel
 *  Initial revision
 *
 **************************************************************************/

#ifndef _MBGSVCTL_H
#define _MBGSVCTL_H


/* Other headers to be included */

#if !defined MBG_OMIT_MBGDEVIO_H
  #include <mbgdevio.h>
#else
  typedef void* MBG_DEVICE_LIST;
#endif

#include <mbg_tgt.h>
#include <use_pack.h>
#include <gpsdefs.h>
#include <timecnv.h>

#if defined( MBG_TGT_WIN32 )
  #include <rs232.h>
  #include <mbgadjtm_cfg.h>
#else
  #define MBGADJTM_CFG int   //##++++++++++++++
  #include <mbgserio.h>
#endif


#ifdef _MBGSVCTL
  #define _ext
#else
  #define _ext extern
#endif


/* Start of header body */

#if defined( _USE_PACK )      // set byte alignment
  #pragma pack( 1 )
#endif


#define MBGSVCTL_VERSION         0x0308

#define MBGSVCTL_COMPAT_VERSION  0x0214


#ifdef __cplusplus
extern "C" {
#endif


/* ----- function prototypes begin ----- */

/* This section was generated automatically */
/* by MAKEHDR, do not remove the comments. */

 /**
 * @brief Get the version number of the precompiled DLL/shared object library.
 *
 * If this library is used as a DLL/shared object library then the version
 * number can be checked to see if the header files which are actually used
 * to build an application are compatible with the header files which have
 * been used to build the library, and thus the API function are called
 * in the correct way.
 *
 * @return The version number
 *
 * @see ::mbgsvctl_check_version
 * @see ::MBGSVCTL_VERSION defined in mbgsvctl.h
 */
 _MBG_API_ATTR int _MBG_API mbgsvctl_get_version( void ) ;

 /**
 * @brief Check if the DLL/shared library is compatible with a given version.
 *
 * If this library is used as a DLL/shared object library then the version
 * number can be checked to see if the header files which are actually used
 * to build an application are compatible with the header files which have
 * been used to build the library, and thus the API functions are called
 * in the correct way.
 *
 * @param[in] header_version  Version number to be checked, should be ::MBGSVCTL_VERSION
 *                            from the mbgsvctl.h file version used to build the application
 *
 * @return ::MBG_SUCCESS if compatible, else ::MBG_ERR_LIB_NOT_COMPATIBLE
 *
 * @see ::mbgsvctl_get_version
 * @see ::MBGSVCTL_VERSION defined in mbgsvctl.h
 */
 _MBG_API_ATTR int _MBG_API mbgsvctl_check_version( int header_version ) ;

 _MBG_API_ATTR void _MBG_API mbgsvctl_log_mbgdevio_error( DWORD ioctl, DWORD sys_rc ) ;
 _MBG_API_ATTR int _MBG_API snprint_system_time( char *s, size_t max_len, const SYSTEMTIME *st ) ;
 /**
 * @brief Register an event source.
 *
 * When an application uses the RegisterEventSource() or OpenEventLog()
 * function to get a handle to an event log, the event logging service
 * searches for the specified source name in the registry. A new source
 * name can be added to the registry by opening a new registry subkey
 * under the application key and adding registry values to the new
 * subkey.
 *
 * @param[in]  eventlog_name  Name of the to be registered as event source,
 *                            e.g. the name of a DLL or application.
 *
 * @return ::MBG_SUCCESS on success, else one of the @ref MBG_ERROR_CODES.
 */
 _MBG_API_ATTR int _MBG_API mbg_svc_register_event_source( const char *eventlog_name ) ;

 _MBG_API_ATTR void _MBG_API mbg_svc_snprint_cfg( char *s, size_t max_len, MBGADJTM_CFG *curr_adjtm_cfg, MBGADJTM_CFG *prev_adjtm_cfg, MBG_SERCLOCK_CFG *curr_serclock_cfg, MBG_SERCLOCK_CFG *prev_serclock_cfg ) ;
 /**
 *@brief Generate a log message according to a message ID.
 *
 * TODO
 *
 * @param[in]  msg_id     One of the message IDs defined in messages.mc, or a system message ID.
 * @param[in]  err_code   ERROR_SUCCESS (==0), or one of the win32 system error codes.
 * @param[in]  fmt        An optional format string, or NULL.
 * @param[in]  info_1     A string parameter, if required by the format string, else NULL.
 * @param[in]  info_2     A second string parameter, if required by the format string, else NULL.
 *
 * @see ::mbg_svc_log_msg
 */
 _MBG_API_ATTR void _MBG_API mbg_svc_log_msg( DWORD msg_id, DWORD sys_rc, const char *fmt, const char *info_1, const char *info_2 ) ;

 /**
 *@brief Generate a log message depending on a Meinberg error code.
 *
 * Unlike the ::mbg_svc_log_msg which expects a message ID as specified
 * e.g. in @ref MBG_WIN_MSG_IDS, this function just expects one of the
 * @ref MBG_ERROR_CODES which is converted to a message ID and passed
 * to ::mbg_svc_log_msg to actually generate the log message.
 *
 * @param[in]  mbg_errno  One of the @ref MBG_ERROR_CODES to be converted to a message ID
 *                        to be passed to ::mbg_svc_log_msg.
 * @param[in]  fmt        An optional format string passed to ::mbg_svc_log_msg.
 * @param[in]  info_1     A string parameter passed to ::mbg_svc_log_msg, depending on the format string
 * @param[in]  info_2     Another string parameter passed to ::mbg_svc_log_msg, depending on the format string
 *
 * @see ::mbg_svc_log_msg
 */
 _MBG_API_ATTR void _MBG_API mbg_svc_log_mbg_error( int mbg_errno, const char *fmt, const char *info_1, const char *info_2 ) ;

 /**
 * @brief Check if the calling process has admin rights.
 *
 * @note The CheckTokenMembership() API call is used if it is supported
 * on the Windows system where the application is running.
 * On legacy systems where the API call is not available we call
 *::legacy_check_token_membership instead.
 *
 * @param[in]  p_sid  Pointer to the Security ID to be checked.
 *
 * @return  true if the calling process has admin rights, else false.
 *
 * @see ::legacy_check_token_membership
 */
 _MBG_API_ATTR BOOL _MBG_API mbg_svc_has_admin_rights( void ) ;

 /**
 * @brief Free a device list that has been set up by ::mbg_svc_find_devices_with_hw_id
 *
 * @param[in,out]  devices  Pointer to the list to be freed.
 *
 * @see ::mbg_svc_find_devices_with_hw_id
 */
 _MBG_API_ATTR void _MBG_API mbg_svc_free_device_list( MBG_DEV_FN_LIST_ENTRY *list ) ;

 _MBG_API_ATTR int _MBG_API mbg_svc_find_devices( void ) ;
 /**
 * @brief Search for Meinberg PnP devices and set up a device list
 *
 * The allocated list can be freed by calling ::mbg_svc_free_device_list.
 *
 * @param[out]  p_list  Pointer to the list to be set up
 * @param[in]   max_devices  Max number of entries to be added to the list  // TODO Make this obsolete
 *
 * @return  The number of Meinberg PnP devices in the list
 */
 _MBG_API_ATTR int _MBG_API mbg_svc_find_devices_with_hw_id( MBG_DEV_FN_LIST_ENTRY **p_list, int max_devices ) ;

 /**
 * @brief Get a device path string for a device from a global list.
 *
 * The list of devices must bhave been set up before
 * by calling ::mbg_find_pnp_devices.
 *
 * @param[in]  device_index  The index of the device in the list.
 *
 * @return  The device path, or NULL if @p device_index is out of range.
 */
 _MBG_API_ATTR const char * _MBG_API mbg_svc_get_device_path( unsigned int device_index ) ;

 /**
 * @brief Write the time service configuration parameters to the registry.
 *
 * @param[in]  p  A structure with the settings to be written to the registry.
 *
 * @return ERROR_SUCCESS, or a win32 system error code.
 */
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_set_mbgadjtm_cfg( const MBGADJTM_CFG *p ) ;

 /**
 * @brief Read the time service configuration parameters from the registry.
 *
 * @param[out]  p  A structure to be filled with the settings read from the registry.
 *
 * @return ERROR_SUCCESS, or a win32 system error code.
 */
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_get_mbgadjtm_cfg( MBGADJTM_CFG *p ) ;

 _MBG_API_ATTR void _MBG_API mbg_svc_get_mbgadjtm_cfg_default( MBGADJTM_CFG *p ) ;
 /**
 * @brief Write the configuration parameters for a serial clock to the registry.
 *
 * @param[in]  p  A structure with the settings to be written to the registry.
 *
 * @return ERROR_SUCCESS, or a win32 system error code.
 */
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_set_mbg_serclock_cfg( const MBG_SERCLOCK_CFG *p ) ;

 _MBG_API_ATTR DWORD _MBG_API mbg_svc_get_mbg_serclock_cfg( MBG_SERCLOCK_CFG *p ) ;
 _MBG_API_ATTR void _MBG_API mbg_svc_get_mbg_serclock_cfg_default( MBG_SERCLOCK_CFG *p ) ;
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_get_msgs_filename( char *s, size_t size ) ;
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_get_stats_filename( char *s, size_t size ) ;
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_get_adjtm_filename( char *s, size_t size ) ;
 /**
 * @brief Query the status of a service with specified name.
 *
 * @param[in]  svc_name  The name of the service to be queried.
 *
 * @return  On success, one of the Windows service status codes
 *          SERVICE_STOPPED, SERVICE_RUNNING, etc., which are
 *          defined in winsvc.h with values > 0.
 *          On error, 0 if the service is not installed, or
 *          (DWORD) -1 if there was a problem querying the status.
 */
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_query_status( const char *svc_name ) ;

 /**
 * @brief Start a named service, log a message on error.
 *
 * @param[in]  svc_name  The name of the service to be started.
 *
 * @return  ERROR_SUCCESS on success, else one of the win32 system error codes.
 *
 * @see ::mbg_svc_stop
 * @see ::mbg_svc_pause
 * @see ::mbg_svc_continue
 */
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_start( const char *svc_name ) ;

 /**
 * @brief Stop a named service, log a message on error.
 *
 * @param[in]  svc_name  The name of the service to be stopped.
 *
 * @return  ERROR_SUCCESS on success, else one of the win32 system error codes.
 *
 * @see ::mbg_svc_start
 * @see ::mbg_svc_pause
 * @see ::mbg_svc_continue
 */
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_stop( const char *svc_name ) ;

 /**
 * @brief Pause a named service, log a message on error.
 *
 * @param[in]  svc_name  The name of the service to be paused.
 *
 * @return  ERROR_SUCCESS on success, else one of the win32 system error codes.
 *
 * @see ::mbg_svc_start
 * @see ::mbg_svc_stop
 * @see ::mbg_svc_continue
 */
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_pause( const char *svc_name ) ;

 /**
 * @brief Start a named service, log a message on error.
 *
 * @param[in]  svc_name  The name of the service to be continued.
 *
 * @return  ERROR_SUCCESS on success, else one of the win32 system error codes.
 *
 * @see ::mbg_svc_start
 * @see ::mbg_svc_stop
 * @see ::mbg_svc_pause
 * @see ::mbg_svc_continue
 */
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_continue( const char *svc_name ) ;

 /**
 * @brief Install a named service, log a message on error.
 *
 * Most parameters are simply passed to CreateService()
 * which is called to actually create the service.
 *
 * @param[in]  svc_name          The short name of the service.
 * @param[in]  svc_type          The service type.
 * @param[in]  svc_start_type    The service start type.
 * @param[in]  svc_disp_name     The long (display) name oif the service.
 * @param[in]  svc_description   A service description string, actually not used.
 * @param[in]  svc_exe_path      Path to the service executable.
 * @param[in]  svc_dependencies  String with names of service on which this one depends.
 *
 * @return ERROR_SUCCESS, or a win32 system error code.
 */
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_install( const char *svc_name, DWORD svc_type, DWORD svc_start_type, const char *svc_disp_name, const char *svc_description, const char *svc_exe_path, const char *svc_dependencies ) ;

 /**
 * @brief Stop and remove a service, log a message on error
 *
 * @param[in]  svc_name  The name of the service to be removed.
 *
 * @return ERROR_SUCCESS, or a win32 system error code.
 */
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_remove( const char *svc_name ) ;

 /**
 * @brief Set a service's start mode to AUTO or MANUALLY.
 *
 * @param[in]  svc_name  The name of the service to be changed.
 *
 * @return ERROR_SUCCESS, or a win32 system error code.
 */
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_set_autostart( const char *svc_name, BOOL autostart ) ;

 _MBG_API_ATTR DWORD _MBG_API mbg_svc_set_description( const char *svc_name, const char *svc_descr ) ;
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_set_ctry_code( const DWORD *p ) ;
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_get_ctry_code( DWORD *p ) ;
 _MBG_API_ATTR void _MBG_API mbg_svc_get_ctry_code_default( DWORD *p ) ;
 _MBG_API_ATTR const char * _MBG_API mbg_svc_name_time_adjustment( void ) ;
 _MBG_API_ATTR const char * _MBG_API mbg_svc_name_clock_kernel_driver( void ) ;
 _MBG_API_ATTR const char * _MBG_API mbg_svc_name_ntp( void ) ;
 /**
 * @brief Get the path of an executable file.
 *
 * @param[out]  path      The output buffer to be filled with the path.
 * @param[in]   max_len   The size of the output buffer AKA maximum path length.
 * @param[in]   exe_name  The base name of the executable.
 * @param[in]   subdir    The subdirectory of the executable.
 *
 * @return ::MBG_SUCCESS on success, else one of the @ref MBG_ERROR_CODES.
 */
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_get_exe_path( char *path, uint32_t max_len, const char *exe_name, const char *subdir ) ;

 _MBG_API_ATTR DWORD _MBG_API mbg_svc_install_time_adjustment( void ) ;
 _MBG_API_ATTR DWORD _MBG_API mbg_svc_uninstall_time_adjustment( void ) ;

/* ----- function prototypes end ----- */

#ifdef __cplusplus
}
#endif


#if defined( _USE_PACK )   // set default alignment
  #pragma pack()
#endif

/* End of header body */

#undef _ext

#endif  // _MBGSVCTL_H

