
/**************************************************************************
 *
 *  $Id: rs232.h 1.13 2018/08/16 12:57:40Z martin REL_M $
 *
 *  Copyright (c) Meinberg Funkuhren, Bad Pyrmont, Germany
 *
 *  Description:
 *    Definitions and prototypes for template.c.
 *
 * -----------------------------------------------------------------------
 *  $Log: rs232.h $
 *  Revision 1.13  2018/08/16 12:57:40Z  martin
 *  Updated function prototypes to avoid compiler warnings.
 *  Revision 1.12  2012/01/12 14:46:06Z  martin
 *  Removed German umlauts in old RCS log entries.
 *  Revision 1.11  2006/11/01 15:42:38Z  martin
 *  Picked up Andre's new method SetComParameter().
 *  Revision 1.10  2006/10/30 14:47:51Z  martin
 *  Updated function prototypes.
 *  Revision 1.9  2004/04/14 08:39:46Z  martin
 *  Pack structures 1 byte aligned.
 *  Revision 1.8  2003/10/21 08:24:29Z  martin
 *  Changed "bool" to "BOOL" to avoid compiler warnings.
 *  Revision 1.7  2003/10/16 12:44:36Z  martin
 *  Don't define class if compiled without _cplusplus.
 *  Revision 1.6  2003/03/27 16:11:17  Daniel
 *  removed obsolete header files
 *  Revision 1.5  2003/03/10 09:26:02Z  Daniel
 *  increased buffer size for ComName
 *  Revision 1.4  2002/11/26 10:24:22Z  martin
 *  Port handle made public.
 *  New variables which keep transmission times per
 *  bit and per char in 100 nsec units, and bits per char.
 *  Use standard file template.
 *  Revision 1.3  2001/09/12 09:50:12Z  Udo
 *  Fehler korrigiert:
 *  Schutz gegen rekursiven Include
 *  undef _ext ergaenzt
 *  Revision 1.2  2001/02/14 11:19:58Z  WERNER
 *  Clear_Error Funktion zugefuegt
 *  Revision 1.1  2000/08/29 07:47:38Z  WERNER
 *  Initial revision
 *
 **************************************************************************/

#ifndef _RS232_H
#define _RS232_H


/* Other headers to be included */

#include <use_pack.h>

#include <windows.h>


#ifdef _RS232
 #define _ext
 #define _DO_INIT
#else
 #define _ext extern
#endif


/* Start of header body */

#if defined( _USE_PACK )      // set byte alignment
  #pragma pack( 1 )
#endif


typedef struct
{
  char ComName[7]; 
  char Framing[4];
  long Baudrate;
} COMSTRUCT;


#ifdef __cplusplus

class Comport{
public:
  HANDLE Handle;
  ULONG  bits_per_char;  // mostly 10 or 11, depending on the framing
  ULONG  bit_time;       // 100 nsec units required to transmit 1 bit and 1 char,
  ULONG  char_time;      // depending on transmission speed and bits per char.

  class  InitException { };
         Comport( COMSTRUCT *Cpnt );          // Konstruktor
         ~Comport(void);                      // Destruktor
         
  void   SetComParameter( COMSTRUCT *Cpnt );
  int    Read( LPSTR lpszBlock, DWORD dwMaxLength );
  int    Write( LPSTR lpszBlock, DWORD dwMaxLength );
  BOOL   RxClear( void );
  BOOL   TxClear( void );
  DWORD  ClearError( void );
};

#endif

/* function prototypes: */

#ifdef __cplusplus
extern "C" {
#endif

/* ----- function prototypes begin ----- */

/* This section was generated automatically */
/* by MAKEHDR, do not remove the comments. */

 HANDLE mbg_open_serial( const char *device_name ) ;
 void mbg_log_com_error( DWORD dwErrorFlags, COMSTAT *pComStat ) ;
 DWORD mbg_clear_comm_error( HANDLE h ) ;
 int mbg_setup_port_list( int max_port_num, void (*callbackfnc)( const char *dev_name ) ) ;

/* ----- function prototypes end ----- */

#ifdef __cplusplus
}
#endif


#if defined( _USE_PACK )   // set default alignment
  #pragma pack()
#endif

/* End of header body */

#undef _ext
#undef _DO_INIT

#endif  /* _RS232_H */

